import { StrictOmit } from "ts-essentials";
export interface URLDataSyncInProps<Data> {
    readonly data: Data | null | undefined;
    readonly dataError: Error | null | undefined;
    readonly isLoadingData: boolean;
    readonly urlQuery: URLQueryMap;
    getData(): void;
    saveData(): void;
    updateData(data: Partial<Data>): void;
    /** Update URL query parameters without reloading and trigger a re-sync. */
    updateURLQuery(query: URLQueryMap): void;
    /** Instead of setting URL query, append to existing URL query. */
    appendURLQuery(query: URLQueryMap): void;
}
export interface URLDataSyncOutProps<Data> {
    readonly urlDataSync: Repository.URLDataSync;
}
/**
 * Automatically sync with current URL by requesting data from server using
 * said URL. This is assuming there is data provided by the server at current
 * URL, e.g. user navigates to /users/1, this will send a GET request to
 * /users/1, which should have a defined backend route that contains the
 * relevant data.
 *
 * This HOC is usually used for components rendered by a Route. Please make sure
 * when implementing the backend to handle these requests that it never returns
 * null/undefined (as per REST design standards).
 */
export declare function urlDataSync<Data, OutProps = {}>(): FunctionalEnhancer<URLDataSyncInProps<Data> & OutProps, URLDataSyncOutProps<Data> & OutProps>;
export interface CursorPaginatedData<T> {
    readonly results: readonly T[];
    readonly limit?: number;
    readonly order?: string;
    readonly sortField?: Extract<keyof T, string>;
    readonly next?: string;
    readonly previous?: string;
    readonly hasNext?: boolean;
    readonly hasPrevious?: boolean;
}
export interface CursorPaginationInProps<T> {
    readonly next?: string;
    readonly hasNext?: boolean;
    readonly previous?: string;
    readonly hasPrevious?: string;
    setNext(next?: string): void;
    setHasNext(hasNext?: boolean): void;
    setPrevious(previous?: string): void;
    setHasPrevious(previous?: boolean): void;
}
/**
 * This works with the auto-sync HOC to provide pagination state (but does not
 * trigger the sync). It is assumed that the server will return the data in the
 * above format - the cursor markers will be stored internally and fed the next
 * time we perform a GET request.
 */
export declare function cursorPagination<T, OutProps = {}>(): FunctionalEnhancer<CursorPaginationInProps<T> & OutProps, OutProps>;
export interface URLCursorPaginatedDataSyncInProps<T> extends StrictOmit<URLDataSyncInProps<readonly T[]>, "data">, Pick<CursorPaginatedData<T>, "limit" | "order" | "sortField"> {
    readonly data: readonly T[];
    goToNextPage(): void;
    goToPreviousPage(): void;
}
export interface URLCursorPaginatedDataSyncOutProps<T> extends Pick<URLDataSyncOutProps<CursorPaginatedData<T>>, "urlDataSync"> {
}
/**
 * This HOC automatically manages pagination data sync, and is best used to
 * display table data. For other kinds of data use the data sync HOC.
 */
export declare function urlCursorPaginatedDataSync<T>(): FunctionalEnhancer<URLCursorPaginatedDataSyncInProps<T>, URLCursorPaginatedDataSyncOutProps<T>>;
//# sourceMappingURL=dataHOC.d.ts.map